<?php

namespace Drupal\Tests\oai_pmh_harvester\Unit;

use Drupal;
use Drupal\Core\DependencyInjection\ContainerBuilder;
use Drupal\oai_pmh_harvester\Util;
use Drupal\Tests\UnitTestCase;
use Exception;
use InvalidArgumentException;

/**
 * Tests the Util class.
 *
 * @coversDefaultClass \Drupal\oai_pmh_harvester\Util
 * @group oai_pmh_harvester
 */
class UtilTest extends UnitTestCase {

  /**
   * Test the Util::interpretDateTime function.
   *
   * @covers ::interpretDateTime
   * @dataProvider dataForInterpretDateTime
   *
   * @param string $value
   * @param int|null $expectedUnixTimestamp
   * @param string|null $expectedExceptionType
   */
  public function testInterpretDateTime(
    string $value,
    ?int $expectedUnixTimestamp,
    ?string $expectedExceptionType
  ) {
    try {
      $d = Util::interpretDateTime($value);
    }
    catch (Exception $e) {
      if (!$expectedExceptionType) {
        $this->fail($e->getMessage());
      }
      $this->assertInstanceOf($expectedExceptionType, $e);
      return;
    }

    // The time zone must always be UTC.
    $this->assertEquals(0, $d->getOffset());

    $this->assertEquals($expectedUnixTimestamp, $d->getTimestamp());
  }

  /**
   * Data provider for `testInterpretDateTime`.
   */
  public function dataForInterpretDateTime(): array {
    return [
      "The correct OAI-PMH datetime format." => [
        "2021-01-01T13:00:00Z",
        1609506000,
        NULL,
      ],
      "Valid ISO8601, but using '+00:00' instead of 'Z'." => [
        "2021-01-01T13:00:00+00:00",
        1609506000,
        NULL,
      ],
      "Using SQL date format, omitting the time zone." => [
        "2021-01-01 13:00:00",
        NULL,
        InvalidArgumentException::class,
      ],
      "Garbage." => [
        "random stuff",
        NULL,
        InvalidArgumentException::class,
      ],
    ];
  }

  public function testUtcRequestTime() {
    // Mock the Drupal::time container.
    $container = new ContainerBuilder();
    Drupal::setContainer($container);
    $mockTime = $this->getMockBuilder('Drupal\Component\Datetime\TimeInterface')
        ->disableOriginalConstructor()
        ->getMock();
    $mockTime->method('getRequestTime')->willReturn(12345);
    $container->set('datetime.time', $mockTime);

    $d = Util::utcRequestTime();
    $this->assertEquals(0, $d->getOffset());
    $this->assertEquals(12345, $d->getTimestamp());
  }

}
